@@ -1001,6 +1001,6 @@ assign("sign", `Simplify.sign`, envir=simplifications)
 assign("if", `Simplify.if`, envir=simplifications)
 assign("besselI", `Simplify.bessel`, envir=simplifications)
 assign("besselK", `Simplify.bessel`, envir=simplifications)
-assign("<-", `Simplify.=`, envir=simplifications)
-assign("=", `Simplify.=`, envir=simplifications)
+#assign("<-", `Simplify.=`, envir=simplifications)
+#assign("=", `Simplify.=`, envir=simplifications)
 assign("{", `Simplify.{`, envir=simplifications)
